@@ -113,6 +113,14 @@ get_builtin_columns (void)
 					       "label", _("MIME Type"),
 					       "description", _("The mime type of the file."),
 					       NULL));
+	columns = g_list_append (columns,
+				 g_object_new (NAUTILUS_TYPE_COLUMN,
+					       "name", "selinux_context",
+					       "attribute", "selinux_context",
+					       "label", _("SELinux Context"),
+					       "description", _("The SELinux security context of the file."),
+					       NULL));
+	
 	return columns;
 }
 
