@@ -85,6 +85,11 @@ typedef struct {
 	} callback;
 	gpointer callback_data;
 	Request request;
+	gboolean active; /* Set to FALSE when the callback is triggered and
+			  * scheduled to be called at idle, its still kept
+			  * in the list so we can kill it when the file
+			  * goes away.
+			  */
 } ReadyCallback;
 
 typedef struct {
@@ -144,7 +149,6 @@ static void     move_file_to_extension_queue    (NautilusDirectory *directory,
 static void     nautilus_directory_invalidate_file_attributes (NautilusDirectory      *directory,
 							       NautilusFileAttributes  file_attributes);
 
-
 void
 nautilus_set_kde_trash_name (const char *trash_dir)
 {
@@ -855,20 +859,29 @@ dequeue_pending_idle_callback (gpointer callback_data)
 		/* check if the file already exists */
 		file = nautilus_directory_find_file_by_name (directory, file_info->name);
 		if (file != NULL) {
-			/* file already exists, check if it changed */
+			/* file already exists in dir, check if we still need to
+			 *  emit file_added or if it changed */
 			set_file_unconfirmed (file, FALSE);
-			if (nautilus_file_update_info (file, file_info, FALSE)) {
+			if (!file->details->is_added) {
+				/* We consider this newly added even if its in the list.
+				 * This can happen if someone called nautilus_file_get()
+				 * on a file in the folder before the add signal was
+				 * emitted */
+				nautilus_file_ref (file);
+				file->details->is_added = TRUE;
+				added_files = g_list_prepend (added_files, file);
+			} else if (nautilus_file_update_info (file, file_info, FALSE)) {
 				/* File changed, notify about the change. */
 				nautilus_file_ref (file);
 				changed_files = g_list_prepend (changed_files, file);
 			}
-			nautilus_file_ref (file);			
 		} else {
 			/* new file, create a nautilus file object and add it to the list */
 			file = nautilus_file_new_from_info (directory, file_info);
 			nautilus_directory_add_file (directory, file);			
+			file->details->is_added = TRUE;
+			added_files = g_list_prepend (added_files, file);
 		}
-		added_files = g_list_prepend (added_files, file);
 	}
 
 	/* If we are done loading, then we assume that any unconfirmed
@@ -883,9 +896,8 @@ dequeue_pending_idle_callback (gpointer callback_data)
 			if (file->details->unconfirmed) {
 				nautilus_file_ref (file);
 				changed_files = g_list_prepend (changed_files, file);
-
-				file->details->is_gone = TRUE;
-				nautilus_directory_remove_file (directory, file);
+				
+				nautilus_file_mark_gone (file);
 			}
 		}
 	}
@@ -1189,6 +1201,7 @@ ready_callback_key_compare (gconstpointer a, gconstpointer b)
 
 	callback_a = a;
 	callback_b = b;
+
 	if (callback_a->file < callback_b->file) {
 		return -1;
 	}
@@ -1225,6 +1238,21 @@ ready_callback_key_compare (gconstpointer a, gconstpointer b)
 	return 0;
 }
 
+static int
+ready_callback_key_compare_only_active (gconstpointer a, gconstpointer b)
+{
+	const ReadyCallback *callback_a;
+
+	callback_a = a;
+
+	/* Non active callbacks never match */
+	if (!callback_a->active) {
+		return -1;
+	}
+
+	return ready_callback_key_compare (a, b);
+}
+
 static void
 ready_callback_call (NautilusDirectory *directory,
 		     const ReadyCallback *callback)
@@ -1269,6 +1297,7 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 	g_assert (file != NULL || directory_callback != NULL);
 	
 	/* Construct a callback object. */
+	callback.active = TRUE;
 	callback.file = file;
 	if (file == NULL) {
 		callback.callback.directory = directory_callback;
@@ -1288,7 +1317,7 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 	/* Check if the callback is already there. */
 	if (g_list_find_custom (directory->details->call_when_ready_list,
 				&callback,
-				ready_callback_key_compare) != NULL) {
+				ready_callback_key_compare_only_active) != NULL) {
 		if (file_callback != NULL && directory_callback != NULL) {
 			g_warning ("tried to add a new callback while an old one was pending");
 		}
@@ -1377,17 +1406,19 @@ nautilus_directory_cancel_callback_internal (NautilusDirectory *directory,
 	}
 	callback.callback_data = callback_data;
 
-	/* Remove queued callback from the list. */
-	node = g_list_find_custom (directory->details->call_when_ready_list,
-				&callback,
-				ready_callback_key_compare);
-	if (node != NULL) {
-		remove_callback_link (directory, node);
-		/* When we change the ready list we need to sync up metadata monitors. */
-		update_metadata_monitors (directory);
-		
-		nautilus_directory_async_state_changed (directory);
-	}
+	/* Remove all queued callback from the list (including non-active). */
+	do {
+		node = g_list_find_custom (directory->details->call_when_ready_list,
+					   &callback,
+					   ready_callback_key_compare);
+		if (node != NULL) {
+			remove_callback_link (directory, node);
+			/* When we change the ready list we need to sync up metadata monitors. */
+			update_metadata_monitors (directory);
+			
+			nautilus_directory_async_state_changed (directory);
+		}
+	} while (node != NULL);
 }
 
 static guint
@@ -1492,9 +1523,7 @@ nautilus_directory_get_info_for_new_files (NautilusDirectory *directory,
 	gnome_vfs_async_get_file_info
 		(&handle,
 		 vfs_uri_list,
-		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		  | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS),
+		 NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS,
 		 GNOME_VFS_PRIORITY_DEFAULT,
 		 new_files_callback,
 		 directory);
@@ -1523,7 +1552,9 @@ nautilus_async_destroying_file (NautilusFile *file)
 
 		if (callback->file == file) {
 			/* Client should have cancelled callback. */
-			g_warning ("destroyed file has call_when_ready pending");
+			if (callback->active) {
+				g_warning ("destroyed file has call_when_ready pending");
+			}
 			remove_callback_link (directory, node);
 			changed = TRUE;
 		}
@@ -1818,40 +1849,85 @@ request_is_satisfied (NautilusDirectory *directory,
 }
 
 static gboolean
-call_ready_callbacks (NautilusDirectory *directory)
+call_ready_callbacks_at_idle (gpointer callback_data)
 {
-	gboolean called_any;
+	NautilusDirectory *directory;
 	GList *node, *next;
 	ReadyCallback *callback;
 
+	directory = NAUTILUS_DIRECTORY (callback_data);
+	directory->details->call_ready_idle_id = 0;
+
 	callback = NULL;
-	called_any = FALSE;
 	while (1) {
-		/* Check if any callbacks are satisifed and call them if they are. */
+		/* Check if any callbacks are non-active and call them if they are. */
 		for (node = directory->details->call_when_ready_list;
 		     node != NULL; node = next) {
 			next = node->next;
 			callback = node->data;
-			if (request_is_satisfied (directory, callback->file, &callback->request)) {
+			if (!callback->active) {
+				/* Non-active, remove and call */
 				break;
 			}
 		}
 		if (node == NULL) {
-			if (called_any) {
-				/* When we change the ready list we need to sync up metadata monitors. */
-				update_metadata_monitors (directory);
-			}
-			return called_any;
+			break;
 		}
-		
+
 		/* Callbacks are one-shots, so remove it now. */
 		remove_callback_link_keep_data (directory, node);
 		
 		/* Call the callback. */
 		ready_callback_call (directory, callback);
 		g_free (callback);
-		called_any = TRUE;
 	}
+
+	/* When we change the ready list we need to sync up metadata monitors. */
+	update_metadata_monitors (directory);
+
+	nautilus_directory_async_state_changed (directory);
+	
+	return FALSE;
+}
+
+static void
+schedule_call_ready_callbacks (NautilusDirectory *directory)
+{
+	if (directory->details->call_ready_idle_id == 0) {
+		directory->details->call_ready_idle_id
+			= g_idle_add (call_ready_callbacks_at_idle, directory);
+	}
+}
+
+/* Marks all callbacks that are ready as non-active and
+ * calls them at idle time, unless they are removed
+ * before then */
+static gboolean
+call_ready_callbacks (NautilusDirectory *directory)
+{
+	gboolean found_any;
+	GList *node, *next;
+	ReadyCallback *callback;
+
+	found_any = FALSE;
+	
+	/* Check if any callbacks are satisifed and mark them for call them if they are. */
+	for (node = directory->details->call_when_ready_list;
+	     node != NULL; node = next) {
+		next = node->next;
+		callback = node->data;
+		if (callback->active &&
+		    request_is_satisfied (directory, callback->file, &callback->request)) {
+			callback->active = FALSE;
+			found_any = TRUE;
+		}
+	}
+	
+	if (found_any) {
+		schedule_call_ready_callbacks (directory);
+	}
+	
+	return found_any;
 }
 
 /* This checks if there's a request for monitoring the file list. */
@@ -1934,7 +2010,7 @@ read_dot_hidden_file (NautilusDirectory *directory)
 	if (gnome_vfs_get_file_info_uri (
 		dot_hidden_vfs_uri,
 		file_info, 
-		GNOME_VFS_FILE_INFO_DEFAULT) != GNOME_VFS_OK) {
+		GNOME_VFS_FILE_INFO_FOLLOW_LINKS) != GNOME_VFS_OK) {
 		gnome_vfs_uri_unref (dot_hidden_vfs_uri);
 		gnome_vfs_file_info_unref (file_info);
 		return;
@@ -2034,9 +2110,7 @@ start_monitoring_file_list (NautilusDirectory *directory)
 	gnome_vfs_async_load_directory
 		(&directory->details->directory_load_in_progress, /* handle */
 		 directory->details->uri,                         /* uri */
-		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE	          /* options */
-		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		  | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS),
+		 NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS,
 		 DIRECTORY_LOAD_ITEMS_PER_CALLBACK,               /* items_per_notification */
 		 GNOME_VFS_PRIORITY_DEFAULT,
 		 directory_load_callback,                         /* callback */
@@ -2172,7 +2246,8 @@ is_needy (NautilusFile *file,
 	for (node = directory->details->call_when_ready_list;
 	     node != NULL; node = node->next) {
 		callback = node->data;
-		if ((* check_wanted) (&callback->request)) {
+		if (callback->active &&
+		    (* check_wanted) (&callback->request)) {
 			if (callback->file == file) {
 				return TRUE;
 			}
@@ -2776,6 +2851,10 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 	result = results->data;
 
 	if (result->result != GNOME_VFS_OK) {
+		if (result->result == GNOME_VFS_ERROR_NOT_FOUND) {
+			/* mark file as gone */
+			nautilus_file_mark_gone (get_info_file);
+		}
 		get_info_file->details->file_info_is_up_to_date = TRUE;
 		if (get_info_file->details->info != NULL) {
 			gnome_vfs_file_info_unref (get_info_file->details->info);
@@ -2783,14 +2862,6 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 		}
 		get_info_file->details->get_info_failed = TRUE;
 		get_info_file->details->get_info_error = result->result;
-		if (result->result == GNOME_VFS_ERROR_NOT_FOUND) {
-			/* mark file as gone */
-
-			get_info_file->details->is_gone = TRUE;
-			if (get_info_file != directory->details->as_file) {
-				nautilus_directory_remove_file (directory, get_info_file);
-			}
-		}
 	} else {
 		nautilus_file_update_info (get_info_file, result->file_info, has_slow_mime_type);
 	}
@@ -2873,9 +2944,7 @@ file_info_start (NautilusDirectory *directory,
 	fake_list.prev = NULL;
 	fake_list.next = NULL;
 
-	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		| GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		| GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+	options = NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS;
 	if (need_slow_mime) {
 		options |= GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE;
 	}
