@@ -34,7 +34,7 @@
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include "nautilus-emblem-utils.h"
@@ -196,12 +196,12 @@ nautilus_emblem_verify_keyword (GtkWindow *parent_window,
 	if (keyword == NULL || strlen (keyword) == 0) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
 				       _("Sorry, but you must specify a non-blank keyword for the new emblem."), 
-				       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		return FALSE;
 	} else if (!emblem_keyword_valid (keyword)) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
 				       _("Sorry, but emblem keywords can only contain letters, spaces and numbers."), 
-				       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		return FALSE;
 	} else if (is_reserved_keyword (keyword)) {
 		char *error_string;
@@ -212,7 +212,7 @@ nautilus_emblem_verify_keyword (GtkWindow *parent_window,
 		 */
 		error_string = g_strdup_printf (_("Sorry, but there is already an emblem named \"%s\"."), display_name);
 		eel_show_error_dialog (_("Please choose a different emblem name."), error_string,
-				       _("Couldn't install emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		g_free (error_string);
 		return FALSE;
 	} 
@@ -238,9 +238,9 @@ nautilus_emblem_install_custom_emblem (GdkPixbuf *pixbuf,
 		return;
 	}
 
-	dir = g_strdup_printf ("%s/.icons/gnome/48x48/emblems",
+	dir = g_strdup_printf ("%s/.icons/hicolor/48x48/emblems",
 			       g_get_home_dir ());
-	stat_dir = g_strdup_printf ("%s/.icons/gnome",
+	stat_dir = g_strdup_printf ("%s/.icons/hicolor",
 				    g_get_home_dir ());
 
 	vfs_uri = gnome_vfs_uri_new (dir);
@@ -255,8 +255,7 @@ nautilus_emblem_install_custom_emblem (GdkPixbuf *pixbuf,
 	/* save the image */
 	if (eel_gdk_pixbuf_save_to_file (pixbuf, path) != TRUE) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
-				       _("Sorry, unable to save custom emblem."), 
-				       _("Couldn't Install Emblem"), 
+				       _("Sorry, unable to save custom emblem."), 				       
 				       GTK_WINDOW (parent_window));
 		g_free (dir);
 		g_free (stat_dir);
@@ -269,11 +268,12 @@ nautilus_emblem_install_custom_emblem (GdkPixbuf *pixbuf,
 	if (display_name != NULL) {
 		path = g_strdup_printf ("%s/emblem-%s.icon", dir, keyword);
 		file = fopen (path, "w+");
-		
+		g_free (path);
+
 		if (file == NULL) {
 			eel_show_error_dialog (_("The emblem cannot be installed."),
 					       _("Sorry, unable to save custom emblem name."), 
-					       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+					       GTK_WINDOW (parent_window));
 			g_free (stat_dir);
 			g_free (dir);
 			return;
@@ -304,7 +304,7 @@ nautilus_emblem_can_remove_emblem (const char *keyword)
 	char *path;
 	gboolean ret = TRUE;
 	
-	path = g_strdup_printf ("%s/.icons/gnome/48x48/emblems/emblem-%s.png",
+	path = g_strdup_printf ("%s/.icons/hicolor/48x48/emblems/emblem-%s.png",
 				g_get_home_dir (), keyword);
 
 	if (access (path, F_OK|W_OK) != 0) {
@@ -322,7 +322,7 @@ nautilus_emblem_can_rename_emblem (const char *keyword)
 	char *path;
 	gboolean ret = TRUE;
 	
-	path = g_strdup_printf ("%s/.icons/gnome/48x48/emblems/emblem-%s.png",
+	path = g_strdup_printf ("%s/.icons/hicolor/48x48/emblems/emblem-%s.png",
 				g_get_home_dir (), keyword);
 
 	if (access (path, F_OK|R_OK) != 0) {
@@ -343,9 +343,9 @@ nautilus_emblem_remove_emblem (const char *keyword)
 	struct utimbuf ubuf;
 	
 	 
-	dir = g_strdup_printf ("%s/.icons/gnome/48x48/emblems",
+	dir = g_strdup_printf ("%s/.icons/hicolor/48x48/emblems",
 			       g_get_home_dir ());
-	stat_dir = g_strdup_printf ("%s/.icons/gnome",
+	stat_dir = g_strdup_printf ("%s/.icons/hicolor",
 				    g_get_home_dir ());
 
 	path = g_strdup_printf ("%s/emblem-%s.png", dir, keyword);
@@ -392,9 +392,9 @@ nautilus_emblem_rename_emblem (const char *keyword, const char *name)
 	struct utimbuf ubuf;
 	FILE *file;
 	
-	dir = g_strdup_printf ("%s/.icons/gnome/48x48/emblems",
+	dir = g_strdup_printf ("%s/.icons/hicolor/48x48/emblems",
 			       g_get_home_dir ());
-	stat_dir = g_strdup_printf ("%s/.icons/gnome",
+	stat_dir = g_strdup_printf ("%s/.icons/hicolor",
 				    g_get_home_dir ());
 
 	path = g_strdup_printf ("%s/emblem-%s.icon", dir, keyword);
