@@ -500,7 +500,26 @@ should_show_file_on_screen (FMDirectoryView *view, NautilusFile *file)
 static void
 fm_icon_view_remove_file (FMDirectoryView *view, NautilusFile *file, NautilusDirectory *directory)
 {
-	g_assert (directory == fm_directory_view_get_model (view));
+	/* This used to assert that 'directory == fm_directory_view_get_model (view)', but that
+	 * resulted in a lot of crash reports (bug #352592). I don't see how that trace happens.
+	 * It seems that somehow we get a files_changed event sent to the view from a directory
+	 * that isn't the model, but the code disables the monitor and signal callback handlers when
+	 * changing directories. Maybe we can get some more information when this happens.
+	 * Further discussion in bug #368178.
+	 */
+	if (directory != fm_directory_view_get_model (view)) {
+		char *file_uri, *dir_uri, *model_uri;
+		file_uri = nautilus_file_get_uri (file);
+		dir_uri = nautilus_directory_get_uri (directory);
+		model_uri = nautilus_directory_get_uri (fm_directory_view_get_model (view));
+		g_warning ("fm_icon_view_remove_file() - directory not icon view model, shouldn't happen.\n"
+			   "file: %p:%s, dir: %p:%s, model: %p:%s, view loading: %d\n"
+			   "If you see this, please add this info to http://bugzilla.gnome.org/show_bug.cgi?id=368178",
+			   file, file_uri, directory, dir_uri, fm_directory_view_get_model (view), model_uri, fm_directory_view_get_loading (view));
+		g_free (file_uri);
+		g_free (dir_uri);
+		g_free (model_uri);
+	}
 	
 	if (nautilus_icon_container_remove (get_icon_container (FM_ICON_VIEW (view)),
 					    NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
@@ -512,22 +531,12 @@ static gboolean
 file_has_lazy_position (FMDirectoryView *view,
 			NautilusFile *file)
 {
-	gboolean lazy_position;
-
 	/* For volumes (i.e. cdrom icon) we use lazy positioning so that when
 	 * an old cdrom gets re-mounted in a place that now has another
-	 * icon we don't overlap that one. We don't do this in general though,
-	 * as it can cause icons moving around.
+	 * icon we don't overlap that one.
 	 */
-	lazy_position = nautilus_file_has_volume (file);
-	if (lazy_position && fm_directory_view_get_loading (view)) {
-		/* if volumes are loaded during directory load, don't mark them
-		 * as lazy. This is wrong for files that were mounted during user
-		 * log-off, but it is right for files that were mounted during login. */
-		lazy_position = FALSE;
-	}
 
-	return lazy_position;
+	return NAUTILUS_IS_DESKTOP_ICON_FILE (file);
 }
 
 static void
@@ -1073,6 +1082,8 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	file = fm_directory_view_get_directory_as_file (view);
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 
+	nautilus_icon_container_set_is_reloading (NAUTILUS_ICON_CONTAINER (icon_container), TRUE);
+
 	nautilus_icon_container_set_allow_moves (NAUTILUS_ICON_CONTAINER (icon_container),
 						 fm_directory_view_get_allow_moves (view));
 
@@ -1147,6 +1158,7 @@ fm_icon_view_end_loading (FMDirectoryView *view)
 	FMIconView *icon_view;
 
 	icon_view = FM_ICON_VIEW (view);
+	nautilus_icon_container_set_is_reloading (get_icon_container (icon_view), FALSE);
 }
 
 static NautilusZoomLevel
@@ -1810,8 +1822,6 @@ play_file (gpointer callback_data)
 					READ_CHUNK_SIZE,
 					&bytes_read);
 			if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-				g_free (buffer);
-				gnome_vfs_close (handle);
 				break;
 			}
 
