@@ -172,6 +172,8 @@ nautilus_application_instance_init (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
+	application->shell = nautilus_shell_new (application);
+	
 	/* Watch for volume mounts so we can restore open windows
 	 * This used to be for showing new window on mount, but is not
 	 * used anymore */
@@ -226,6 +228,11 @@ nautilus_application_destroy (BonoboObject *object)
 	
 	g_object_unref (application->undo_manager);
 
+	if (application->shell_registered) {
+		bonobo_activation_unregister_active_server (SHELL_IID, BONOBO_OBJREF (application->shell));
+	}
+	bonobo_object_unref (application->shell);
+
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
 
@@ -517,12 +524,16 @@ nautilus_application_startup (NautilusApplication *application,
 	while (TRUE) {
 		/* Try to register the file manager view factory. */
 		result = nautilus_bonobo_activation_register_for_display
-			(FACTORY_IID, BONOBO_OBJREF (application));
+			(SHELL_IID, BONOBO_OBJREF (application->shell));
 
 		switch (result) {
 		case Bonobo_ACTIVATION_REG_SUCCESS:
 			/* We are registered and all is right with the world. */
+			application->shell_registered = TRUE;
 			finish_startup (application);
+			message = NULL;
+			detailed_message = NULL;
+			break;
 		case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE:
 			/* Another copy of nautilus already is running and registered. */
 			message = NULL;
@@ -583,7 +594,7 @@ nautilus_application_startup (NautilusApplication *application,
 
 		/* Get the shell object. */
 		if (message == NULL) {
-			shell = bonobo_activation_activate_from_id (SHELL_IID, 0, NULL, NULL);
+			shell = bonobo_activation_activate_from_id (SHELL_IID, Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, NULL);
 			if (!CORBA_Object_is_nil (shell, &ev)) {
 				break;
 			}
@@ -667,16 +678,6 @@ nautilus_application_startup (NautilusApplication *application,
 		init_session ();
 	}
 
-	/* We're done with the shell now, so let it go. */
-	/* HACK: Don't bother releasing the shell in the case where we
-	 * just told it to quit -- that just leads to hangs and does
-	 * no good. We could probably fix this in some fancier way if
-	 * we could figure out a better lifetime rule.
-	 */
-	if (!(kill_shell || restart_shell)) {
-		bonobo_object_release_unref (shell, NULL);
-	}
-
  out:
 	CORBA_exception_free (&ev);
 }
@@ -802,6 +803,9 @@ nautilus_application_open_desktop (NautilusApplication *application)
 {
 	if (nautilus_application_desktop_windows == NULL) {
 		nautilus_application_create_desktop_windows (application);
+		
+		/* Make sure we update the session when the desktop is created */
+		update_session (gnome_master_client ());
 	}
 }
 
@@ -813,6 +817,9 @@ nautilus_application_close_desktop (void)
 				(GFunc) gtk_widget_destroy, NULL);
 		g_list_free (nautilus_application_desktop_windows);
 		nautilus_application_desktop_windows = NULL;
+
+		/* Make sure we update the session when the desktop goes away */
+		update_session (gnome_master_client ());
 	}
 }
 
@@ -1257,11 +1264,6 @@ desktop_changed_callback (gpointer user_data)
 	} else {
 		nautilus_application_close_desktop ();
 	}
-
-	/* Can't make this function just watch the preference
-	 * itself changing since ordering is important
-	 */
-	update_session (gnome_master_client ());
 }
 
 static gboolean
@@ -1358,7 +1360,7 @@ volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
 static void
 removed_from_session (GnomeClient *client, gpointer data)
 {
-	nautilus_main_event_loop_quit ();
+	nautilus_main_event_loop_quit (FALSE);
 }
 
 static char *
