@@ -43,7 +43,7 @@
 #include <gtk/gtkpaned.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-types.h>
@@ -852,13 +852,17 @@ command_button_callback (GtkWidget *button, char *id_str)
 {
 	NautilusInformationPanel *information_panel;
 	GnomeVFSMimeApplication *application;
+	GList files;
 	
 	information_panel = NAUTILUS_INFORMATION_PANEL (g_object_get_data (G_OBJECT (button), "user_data"));
 
 	application = gnome_vfs_mime_application_new_from_desktop_id (id_str);
 
 	if (application != NULL) {
-		nautilus_launch_application (application, information_panel->details->file,
+		files.next = NULL;
+		files.prev = NULL;
+		files.data = information_panel->details->file;
+		nautilus_launch_application (application, &files,
 					     nautilus_information_panel_get_window (information_panel));	
 
 		gnome_vfs_mime_application_free (application);
@@ -904,9 +908,12 @@ add_command_buttons (NautilusInformationPanel *information_panel, GList *applica
 
 		id_string = g_strdup (gnome_vfs_mime_application_get_desktop_id (application));
 
-		eel_gtk_signal_connect_free_data 
-			(GTK_OBJECT (temp_button), "clicked",
-			 G_CALLBACK (command_button_callback), id_string);
+		g_signal_connect_data (temp_button,
+				       "clicked",
+				       G_CALLBACK (command_button_callback),
+				       id_string,
+				       (GClosureNotify)g_free,
+				       0);
 
                 g_object_set_data (G_OBJECT (temp_button), "user_data", information_panel);
 		
@@ -948,10 +955,14 @@ add_buttons_from_metadata (NautilusInformationPanel *information_panel, const ch
 					information_panel->details->has_buttons = TRUE;
 					command_string = g_strdup (temp_str + 1);
 					g_free (button_name);
-					
-					eel_gtk_signal_connect_free_data 
-						(GTK_OBJECT (temp_button), "clicked",
-						 G_CALLBACK (metadata_button_callback), command_string);
+
+					g_signal_connect_data (temp_button,
+							       "clicked",
+							       G_CALLBACK (metadata_button_callback),
+							       command_string,
+							       (GClosureNotify)g_free,
+							       0);
+
 		                	g_object_set_data (G_OBJECT (temp_button), "user_data", information_panel);
 					
 					gtk_widget_show (temp_button);			
