@@ -48,11 +48,15 @@
 #include <glib/gi18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-client.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libnautilus-private/nautilus-directory-metafile.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
 #include <libxml/parser.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -105,6 +109,15 @@ event_loop_unregister (GtkObject *object)
 	}
 }
 
+static gboolean
+initial_event_loop_needed (gpointer data)
+{
+	if (!is_event_loop_needed ()) {
+		eel_gtk_main_quit_all ();
+	}
+	return FALSE;
+}
+
 void
 nautilus_main_event_loop_register (GtkObject *object)
 {
@@ -120,8 +133,13 @@ nautilus_main_is_event_loop_mainstay (GtkObject *object)
 }
 
 void
-nautilus_main_event_loop_quit (void)
+nautilus_main_event_loop_quit (gboolean explicit)
 {
+	if (explicit) {
+		/* Explicit --quit, make sure we don't restart */
+		gnome_client_set_restart_style (gnome_master_client (),
+						GNOME_RESTART_IF_RUNNING);
+	}
 	while (event_loop_registrants != NULL) {
 		gtk_object_destroy (event_loop_registrants->data);
 	}
@@ -224,6 +242,8 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+	setlocale (LC_ALL, "");
+
 	if (g_getenv ("NAUTILUS_DEBUG") != NULL) {
 		eel_make_warnings_and_criticals_stop_in_debugger ();
 	}
@@ -362,11 +382,18 @@ main (int argc, char *argv[])
 			 remaining);
 		g_free (startup_id_copy);
 
-		if (is_event_loop_needed ()) {
-			gtk_main ();
-		}
+		/* The application startup does things in an idle, so
+		   we need to check whether the main loop is needed in an idle
+		*/
+		g_idle_add (initial_event_loop_needed, NULL);
+		gtk_main ();
 	}
 
+	/* This has to be done before gnome_vfs_shutdown, because
+ 	 * it might call nautilus_file_get_uri() which might call
+ 	 * gnome_vfs_uri_append_string()
+ 	 */
+ 	eel_debug_shut_down ();
 
 	gnome_vfs_shutdown ();
 
@@ -377,8 +404,6 @@ main (int argc, char *argv[])
 		bonobo_object_unref (application);
 	}
 
-	eel_debug_shut_down ();
-
 	/* If told to restart, exec() myself again. This is used when
 	 * the program is told to restart with CORBA, for example when
 	 * an update takes place.
@@ -399,8 +424,7 @@ main (int argc, char *argv[])
 		execvp (argv[0], argv_copy);
 	}
 
-	/* Shouldn't we do this? Keeping it commented for now
-	g_object_unref (G_OBJECT (program)); */
+	g_object_unref (G_OBJECT (program));
 
 	return EXIT_SUCCESS;
 }
