@@ -36,7 +36,6 @@
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-information-panel.h"
 #include "nautilus-main.h"
-#include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-zoom-control.h"
@@ -59,7 +58,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
@@ -79,6 +78,7 @@
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-undo.h>
 #include <libnautilus-private/nautilus-search-directory.h>
+#include <libnautilus-private/nautilus-signaller.h>
 #include <math.h>
 #include <sys/time.h>
 
@@ -159,7 +159,6 @@ nautilus_window_init (NautilusWindow *window)
 	gtk_widget_show (table);
 	gtk_container_add (GTK_CONTAINER (window), table);
 
-
 	statusbar = gtk_statusbar_new ();
 	window->details->statusbar = statusbar;
 	gtk_table_attach (GTK_TABLE (table),
@@ -191,6 +190,10 @@ nautilus_window_init (NautilusWindow *window)
 				 G_CALLBACK (icons_changed_callback), window,
 				 0);
 
+	/* Register to menu provider extension signal managing menu updates */
+	g_signal_connect_object (nautilus_signaller_get_current (), "popup_menu_changed",
+			 G_CALLBACK (nautilus_window_load_extension_menus), window, G_CONNECT_SWAPPED);
+
 	gtk_quit_add_destroy (1, GTK_OBJECT (window));
 
 	/* Keep the main event loop alive as long as the window exists */
@@ -642,6 +645,25 @@ nautilus_window_finalize (GObject *object)
 	G_OBJECT_CLASS (nautilus_window_parent_class)->finalize (object);
 }
 
+static GObject *
+nautilus_window_constructor (GType type,
+			     guint n_construct_properties,
+			     GObjectConstructParam *construct_params)
+{
+	GObject *object;
+	NautilusWindow *window;
+
+	object = (* G_OBJECT_CLASS (nautilus_window_parent_class)->constructor) (type,
+										 n_construct_properties,
+										 construct_params);
+
+	window = NAUTILUS_WINDOW (object);
+
+	nautilus_window_initialize_menus_constructed (window);
+
+	return object;
+}
+
 void
 nautilus_window_show_window (NautilusWindow *window)
 {
@@ -1334,7 +1356,7 @@ remove_from_history_list (NautilusBookmark *bookmark)
 	}
 }
 
-static void
+static gboolean
 add_to_history_list (NautilusBookmark *bookmark)
 {
 	/* Note that the history is shared amongst all windows so
@@ -1371,8 +1393,10 @@ add_to_history_list (NautilusBookmark *bookmark)
 			}
 		}
 
-		nautilus_send_history_list_changed ();
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 void
@@ -1385,12 +1409,30 @@ nautilus_remove_from_history_list_no_notify (const char *uri)
 	g_object_unref (bookmark);
 }
 
+gboolean
+nautilus_add_to_history_list_no_notify (const char *uri,
+					const char *name,
+					gboolean has_custom_name,
+					const char *icon)
+{
+	NautilusBookmark *bookmark;
+	gboolean ret;
+
+	bookmark = nautilus_bookmark_new_with_icon (uri, name, has_custom_name, icon);
+	ret = add_to_history_list (bookmark);
+	g_object_unref (bookmark);
+
+	return ret;
+}
+
 static void
 real_add_current_location_to_history_list (NautilusWindow *window)
 {
         g_assert (NAUTILUS_IS_WINDOW (window));
                 
-        add_to_history_list (window->current_location_bookmark);
+	if (add_to_history_list (window->current_location_bookmark)) {
+		nautilus_send_history_list_changed ();
+	}
 }
 
 void
@@ -1532,8 +1574,9 @@ static void
 nautilus_window_class_init (NautilusWindowClass *class)
 {
 	GtkBindingSet *binding_set;
-	
+
 	G_OBJECT_CLASS (class)->finalize = nautilus_window_finalize;
+	G_OBJECT_CLASS (class)->constructor = nautilus_window_constructor;
 	G_OBJECT_CLASS (class)->get_property = nautilus_window_get_property;
 	G_OBJECT_CLASS (class)->set_property = nautilus_window_set_property;
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_window_destroy;
@@ -1603,3 +1646,19 @@ nautilus_window_class_init (NautilusWindowClass *class)
 			     "    widget \"*.nautilus-extra-view-widget\" style:rc \"nautilus-extra-view-widgets-style-internal\" \n"
 			     "\n");
 }
+
+/**
+ * nautilus_window_has_menubar_and_statusbar:
+ * @window: A #NautilusWindow
+ * 
+ * Queries whether the window should have a menubar and statusbar, based on the
+ * window_type from its class structure.
+ * 
+ * Return value: TRUE if the window should have a menubar and statusbar; FALSE
+ * otherwise.
+ **/
+gboolean
+nautilus_window_has_menubar_and_statusbar (NautilusWindow *window)
+{
+	return (nautilus_window_get_window_type (window) != NAUTILUS_WINDOW_DESKTOP);
+}
